define([
    'ko',
    'jquery',
    'uiComponent',
    'uiRegistry',
    'mage/translate',
    'Magento_Checkout/js/model/quote',
    'Bss_OneStepCheckout/js/action/validate-shipping-information',
    'Bss_OneStepCheckout/js/action/validate-gift-wrap-before-order',
    'Magento_Checkout/js/model/full-screen-loader',
    'Magento_Checkout/js/action/select-billing-address',
    'Magento_Checkout/js/model/payment/additional-validators',
    'Magento_Checkout/js/model/shipping-service',
    'underscore',
    'Magento_Ui/js/modal/alert',
    'Magento_Checkout/js/checkout-data',
    'Magento_Checkout/js/model/totals'
], function (
    ko,
    $,
    Component,
    registry,
    $t,
    quote,
    validateShippingInformationAction,
    validateGiftWrapAction,
    fullScreenLoader,
    selectBillingAddress,
    additionalValidators,
    shippingService,
    _,
    alert,
    checkoutData,
    totals
) {
    'use strict';
    return function (PlaceOrder) {
        return PlaceOrder.extend({
            placeOrder: function (data, event) {
                var self = this;
                var shippingAddressComponent = registry.get('checkout.steps.shipping-step.shippingAddress');
                var billingAddressComponent = registry.get('checkout.steps.billing-step.payment.payments-list.billing-address-form-shared');

                if (event) {
                    event.preventDefault();
                }

                if (additionalValidators.validate()) {
                    var buttonPlaceOrder =  $('input#' + self.getCode()).closest('.payment-method').find('.payment-method-content .actions-toolbar:not([style*="display: none"]) button.action.checkout');
                    var selectedPaymentMethod = checkoutData.getSelectedPaymentMethod();
                    if (quote.isVirtual()) {
                        if (selectedPaymentMethod.indexOf('sagepay') != -1) {
                            buttonPlaceOrder.not('[style*="display: none"]').trigger('click');
                        } else {
                            buttonPlaceOrder.trigger('click');
                        }
                    } else {
                        if (shippingAddressComponent.validateShippingInformation()) {
                            if (billingAddressComponent.isAddressSameAsShipping()) {
                                fullScreenLoader.startLoader();
                                selectBillingAddress(quote.shippingAddress());
                            }
                            validateShippingInformationAction().done(
                                function () {
                                    var action = 0;
                                    if ($('#giftwrap-checkbox input[name="giftwrap"]').is(":checked")) {
                                        action = 1;
                                    }
                                    var fee = $('.giftwrap .amount .price').attr('amount');
                                    if (typeof fee === "undefined") {
                                        fee = 0;
                                    }
                                    fee = parseFloat(fee);
                                    if (fee < 0) {
                                        fee = 0;
                                    }
                                    validateGiftWrapAction(fee, action).done(
                                        function (response) {
                                            var res = JSON.parse(response);
                                            if (res.status == false || res.gift_wrap_update == true || (res.gift_wrap_display == false && $('#giftwrap-checkbox').length)) {
                                                return;
                                            }
                                            
                                            if (selectedPaymentMethod == "braintree_paypal" || selectedPaymentMethod.indexOf('sagepay') != -1) {
                                                fullScreenLoader.stopLoader();
                                                totals.isLoading(false);
                                            }

                                            if (selectedPaymentMethod.indexOf('sagepay') != -1) {
                                                buttonPlaceOrder.not('[style*="display: none"]').trigger('click');
                                            } else {
                                                buttonPlaceOrder.trigger('click');
                                            }
                                        }
                                    );
                                }
                            ).fail(
                                function () {
                                    fullScreenLoader.stopLoader();
                                }
                            );
                        }
                    }
                }
                return false;
            },
        });
    }
});